#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include "contrast.h"
#include "edges.h"


void norm_dbl(double *from, double *to, int size, double max)
{
  int i;
  double factor=255.0/max;
  FILE *fa, *fb;

  for(i=0; i<size; i++)
    to[i] = factor>0.0 ? from[i]*factor : 0.0;
}

void d_to_c(double *from, unsigned char *to, int size)
{
  int i;
  for (i=0; i<size; i++)
    to[i] = from[i]>0.0 ? (unsigned char)from[i] :0;
}


void contrast(unsigned char *inten_map, unsigned char *edge_map[], 
		int width, double **c_map, int c_radius)
{
  int size = width * width;
  int s_radius=c_radius*3;
  unsigned char *char_map;
  double max, max_inten;
  double **orient, *inten;
  double **gaus_c, **gaus_s;
  int i;

  static int level=0;
  char f_name[20];
  unsigned char *temp_c;
  double *temp_d;

  inten = (double *) calloc(size,  sizeof(double));
  orient = (double **) malloc(ORIENTATIONS * sizeof(double *));
  for (i=0; i<ORIENTATIONS; i++)
  {
    orient[i] = (double *) calloc(size, sizeof(double));
  }
  *c_map = (double *) malloc(size * sizeof(double));


  /* allocate memory for gaussians */
  gaus_c=(double **) calloc(2*c_radius+1, sizeof(double*));
  for (i=0; i<2*c_radius+1; i++)
    gaus_c[i]=(double *) calloc(2*c_radius+1, sizeof(double));

  gaus_s=(double **) calloc(2*s_radius+1, sizeof(double*));
  for (i=0; i<2*s_radius+1; i++)
    gaus_s[i]=(double *) calloc(2*s_radius+1, sizeof(double));


  /* calculate values for gaussians */
  Calc_Gaus(gaus_c, c_radius);
  Calc_Gaus(gaus_s, s_radius);

  max=Orientation(width, c_radius, s_radius, gaus_c, gaus_s, edge_map,
		  orient);
  max_inten=Intensity(width, c_radius, s_radius, gaus_c, gaus_s,
			inten_map, inten);

  if (max_inten > max) max = max_inten;
  combine_maps(max, *c_map, orient, inten, size);

/*
  sprintf(f_name, "inten%d.pgm", level);
  temp_c=(unsigned char *) malloc(size);
  temp_d=(double *) malloc(size * sizeof(double));
  norm_dbl(inten, temp_d, size, max);
  d_to_c(temp_d, temp_c, size);
  write_pgm(f_name, temp_c, width, width, 255);

  for(i=0; i<ORIENTATIONS; i++)
  {
    sprintf(f_name, "orien%d.l%d.pgm", i, level);
    norm_dbl(orient[i], temp_d, size, max);
    d_to_c(temp_d, temp_c, size);
    write_pgm(f_name, temp_c, width, width, 255);
  }
  level++;
  free(temp_c);
  free(temp_d);
*/

} /* contrast()*/


void combine_maps(double max, double *values, double **o_contrast,
		  double *i_contrast, int size)
{
  int i, j;
  double tmax, factor;
  unsigned char *temp_c;
  int width=sqrt(size);
  static int level=0;
  char f_name[20];

  factor=255.0/max;

  for (i=0; i<size; i++)
  {
    tmax=i_contrast[i];
    tmax=0;
    for (j=0; j<ORIENTATIONS; j++)
      if (o_contrast[j][i] > tmax)
	tmax=o_contrast[j][i];
    values[i] = (double) (tmax*factor);
  }
/*
  temp_c=(unsigned char *) malloc(size);
  d_to_c(values, temp_c, size);
  sprintf(f_name, "temp%d.pgm", level);
  write_pgm(f_name, temp_c, width, width, 255);
  level++;
*/
} /* combine_maps */

void clear_maps()
{
  int i, j;

/*
  for (i=1; i<pyramid.numHierarchies; i++)
    for (j=0; j<pyramid.levelSize[0]; j++)
      data[i].level[0][0].value[j]=0.0;
*/
} /* clear_maps() */

double Orientation(int width, int center_radius, int radius, 
		   double **gaus_c, double **gaus_s,
		   unsigned char **edges, double **orient_contrast)
{
  int i, j, pixel_x, pixel_y;
  int temp;
  float weight[ORIENTATIONS];
  double cell[ORIENTATIONS];
  double max=0.0;

  Calc_Weights(weight);

  for (pixel_y=radius; pixel_y<width-radius; pixel_y++)
    for (pixel_x=radius; pixel_x<width-radius; pixel_x++)
    {
      for(i=0; i<ORIENTATIONS; i++)
      {
        cell[i]=Convolve(pixel_x, pixel_y, width,
                         center_radius, gaus_c, edges[i]);
        cell[i]=cell[i]-Convolve(pixel_x, pixel_y, width, radius,
                                 gaus_s, edges[i]);
      }

      for(i=0; i<ORIENTATIONS; i++)
      {
        for(j=0; j<ORIENTATIONS; j++)
	{
          orient_contrast[i][pixel_y*width+pixel_x] 
			+= weight[(j-i+ORIENTATIONS)%ORIENTATIONS]
			   * cell[j];
	}
        if (orient_contrast[i][pixel_y*width+pixel_x]>max)
	  max=orient_contrast[i][pixel_y*width+pixel_x];
      } /* for (i) */

    }/* for (pixels) */

  return max;

}/* Orientation()*/


double Intensity(int width, int center_radius, int radius, 
		 double **gaus_c, double **gaus_s,
		 unsigned char *values, double *inten_contrast)
{
  int i, j, pixel_x, pixel_y;
  double cell;
  double max=0.0;

  int size=width*width;
  static int level=0;
  char f_name[20];
  unsigned char *temp_c;
  double *temp_d;
  FILE *f_ptr;

/*  f_ptr=fopen("temp1", "w");
*/
  /* convolve image with difference of gaussians */
  for (pixel_y=radius; pixel_y<width-radius; pixel_y++)
  {
    for (pixel_x=radius; pixel_x<width-radius; pixel_x++)
    {
      cell=Convolve(pixel_x, pixel_y, width, center_radius, gaus_c,
		    values);
      cell=cell-Convolve(pixel_x, pixel_y, width, radius, gaus_s, values);
      inten_contrast[pixel_y*width+pixel_x]=fabs(cell);

/*      fprintf(f_ptr, "%d %d %g\n", pixel_y, pixel_y,
                inten_contrast[pixel_y*width+pixel_x]);*/

      if(inten_contrast[pixel_y*width+pixel_x]>max)
	max = inten_contrast[pixel_y*width+pixel_x];
    }/* for pixel_x */
  } /* for(pixel_y) */

  sprintf(f_name, "values%d.pgm", level);
/*
  temp_c=(unsigned char *) malloc(size);
  temp_d=(double *) malloc(size * sizeof(double));
  norm_dbl(inten_contrast, temp_d, size, max);
  f_ptr=fopen("temp1", "w");
  for (pixel_y=0; pixel_y<width; pixel_y++)
    for (pixel_x=0; pixel_x<width; pixel_x++)
      fprintf(f_ptr, "%d %d %g\n", pixel_y, pixel_x,
                temp_d[pixel_y*width+pixel_x]);
  fclose(f_ptr);
  d_to_c(temp_d, temp_c, size);
  write_pgm(f_name, values, width, width, 255);
  level++;
  free(temp_c);
  free(temp_d);
*/

  return max;

} /* Intensity()*/



void Calc_Gaus(double **gaus, int rad)
{
  int i,j;
  double total=0.0;
  double sigma=rad*1.4;

  for(i=-rad; i<=rad; i++)
    for (j=-rad; j<=rad; j++)
    {
      gaus[i+rad][j+rad]=4*exp(-(float)(i*i+j*j)/(2.0*sigma));
      total+=gaus[i+rad][j+rad];
    }

  for(i=-rad; i<=rad; i++)
    for (j=-rad; j<=rad; j++)
      gaus[i+rad][j+rad]=gaus[i+rad][j+rad]/total;
}/* Calc_Gaus() */

void Calc_Weights(float *w)
{
  int i;
  float denom;

  w[0]=1.;

  denom=ORIENTATIONS*ORIENTATIONS;
  if (ORIENTATIONS%2)
    denom=denom-1;
 

  for (i=1; i<=ORIENTATIONS/2; i++)
  {
    w[i]=(float)(-4*i)/denom;
    w[ORIENTATIONS-i]=w[i];
  }

} /* Calc_Weights() */


double Convolve(int x, int y, int width, int rad, double **gaus,
                unsigned char *values)
{
  int i,j;
  double temp=0;

  for(i=-rad; i<=rad; i++)
    for(j=-rad; j<=rad; j++)
        temp+=(double) values[(y-i)*width+(x-j)] *
                gaus[i+rad][j+rad];

  return temp;
} /* Convolve */
