#include <stdio.h>
#include <math.h>
#include "edges.h"
#include "pgm.h"

double tn[18]={	0.0874886635,	/* 5 */
		0.2679491924,	/* 15 */ 
		0.4663076582,	/* 25 */ 
		0.7002075382,	/* 35 */ 
		1.0,		/* 45 */ 
		1.428148007,	/* 55 */ 
		2.144506921,	/* 65 */ 
		3.732050808,	/* 75 */ 
		11.4300523,	/* 85 */ 
		-11.4300523,	/* 95 */ 
		-3.732050808,	/* 105 */ 
		-2.144506921,	/* 115 */ 
		-1.428148007,	/* 125 */ 
		-1.0,		/* 135 */ 
		-0.7002075382,	/* 145 */ 
		-0.4663076582,	/* 155 */ 
		-0.2679491924,	/* 165 */ 
		-0.0874886635};	/* 175 */ 


main(int argc, char* argv[])
{
  int width, height, max;
  int i, j;
  int size;
  unsigned char *edge_map1[ORIENTATIONS], *edge_map2[ORIENTATIONS];
  unsigned char *image;
  char base[20], fname[35];

  max=read_pgm(argv[1], &image, &width, &height);
  size=width*height;
  write_pgm("temp.pgm", image, width, height, max);

  Edges(width, height, size, edge_map1, image);

  strncpy(base, argv[1], strlen(argv[1])-3);
  base[strlen(argv[1])-3]=0;
  sprintf(fname, "%se000.pgm", base);
  write_pgm(fname, edge_map1[0], width, height, max);
  sprintf(fname, "%se045.pgm", base);
  write_pgm(fname, edge_map1[1], width, height, max);
  sprintf(fname, "%se090.pgm", base);
  write_pgm(fname, edge_map1[2], width, height, max);
  sprintf(fname, "%se135.pgm", base);
  write_pgm(fname, edge_map1[3], width, height, max);
}



void Edges(int width, int height, int size, unsigned char **edge_map,
		unsigned char *image)
{
  int i, j, edge;
  int radius=1;
  double *temp_map[ORIENTATIONS];
  char f_name[20];
  int edges[size];
  double mag[size];
  int start_x=9, start_y=6;

  start_x=start_x*16+16;
  start_y=start_y*16+16;

  for (i=0; i<ORIENTATIONS; i++)
  {
    temp_map[i]=(double *) calloc(size, sizeof(double));
    edge_map[i]=(unsigned char *) calloc(size, 1);
  }

  for (i=radius; i<height-radius; i++)
  {
    for (j=radius; j<width-radius; j++)
    {
      edges[i*width+j]=Sobel(i, j, width, image, mag);
    }
  }

  for (i=0; i<16; i++)
  {
    for (j=0; j<16; j++)
      printf("%d ", edges[(start_y+i)*width+(start_x+j)]);
    printf("\n");
  } 

  create_maps(edges, temp_map, size, mag);

  MultipleNormalize(size, edge_map, temp_map);

}

void create_maps(int *edges, double **maps, int size, double *mag)
{
  int focus, i;
  double diff, factor=255.0/90.0;

  for (focus=0; focus<4; focus++)
    for(i=0; i<size; i++)
    {
      if (edges[i]<0) maps[focus][i]=0;
      else
      {
	diff=fabs(focus*45-edges[i]);
	if (diff>90.0) diff=180-diff;
	maps[focus][i]=(90.0-diff)*factor*mag[i];
      }
    }
  
}

int Sobel(int i, int j, int width, unsigned char *image, double *mag)
{
  float temp;
  double Sx, Sy, ratio;
  int result;

  static int X[9] = {-1,  0,  1,
		     -2,  0,  2,
		     -1,  0,  1 };

  static int Y[9] = {-1, -2, -1,
		      0,  0,  0,
		      1,  2,  1 };

  Sx=Grad(i, j, width, X, image);
  Sy=Grad(i, j, width, Y, image);
  mag[i*width+j]=sqrt(Sx*Sx+Sy*Sy);

  if (Sy==0.0)
  {
    if (Sx==0.0)  result=-1;
    else result=90;
  }
/*  else if (Sx==0.0) result=0;*/
  else
  {
    ratio=Sx/Sy;
    result = rtan(ratio);
  }

/*
  if ((i==7) || (i==8))
    printf("i=%d j=%d Sx=%g Sy=%g ratio=%g result=%d mag=%g\n", i, j, Sx,
		Sy, ratio, result, mag[i*width+j]);
*/
  return result;
}

int rtan(double ratio)
{
  static int first=1;
  int i;

  for (i=9; i<18; i++)
    if(ratio<tn[i]) return (i*10);

  for (i=0; i<9; i++)
    if(ratio<tn[i]) return (i*10);

  return 90;
}

double Grad(int i, int j, int width, int mask[9], unsigned char *image)
{
  int x, y;
  double total=0.0;
  int temp;

  i--; j--;

  for(y=0; y<3; y++)
    for(x=0; x<3; x++)
    {
      temp = mask[y*3+x] * image[(i+y)*width + (j+x)];
      total+=temp;
    }

  return total;

}



void MultipleNormalize(int size, unsigned char **e_map, double **t_map)
{
  int i;
  float currMax, maxVal = 0.0;
  
  for(i=0; i< ORIENTATIONS; i++)
  {
    currMax = Maximum(i, size, t_map);
    if (currMax > maxVal)
       maxVal=currMax;
  }

  for(i=0; i< ORIENTATIONS; i++)

    Normalize(i, maxVal, size, e_map, t_map);

  return;
}  



void Normalize (int edge, int maxValue, int size, unsigned char **e_map,
		double **temp_map)
{
  int i;
  float multFac;

  multFac=255.0/maxValue;

  for(i=0; i<size; i++)
    e_map[edge][i]=(unsigned char)(temp_map[edge][i]*multFac);
}



float Maximum(int edge, int size, double **temp_map)
{
  int i;
  float max=0;

  for (i=0; i<size; i++)
  {
    if (temp_map[edge][i]>max)
      max=temp_map[edge][i];
  }

  return max;
}

