#include<stdio.h>
#include<stdlib.h>
#include "pgm.h"
#include "edges.h"
#include "contrast.h"
#include "test.h"

main(int argc, char *argv[])
{
  int i, j;
  int level_size[LEVELS];

  unsigned char *image;	/* for read pgm */
  int width, height, max;
  int size;
  int elsize, c_rad;

  int *edge_map; /* for edges */
  int *mag;
  int *inten_map;

  int *edge_tree[LEVELS]; /* for tree */
  int *inten_tree[LEVELS];
  int *mag_tree[LEVELS];
  int *one_tree[LEVELS];

  double *dbl_map[LEVELS];     /* for contrast */
  unsigned char *c_map;
  double max_con, t_max;
  int max_ind;
  
  int temp, lev, orien, x, y;
  char f_name[20];
  FILE *out_o, *out_m;
  int el_size;

  printf("\n%s\n", argv[1]);

  /* read pgm file */
  max = read_pgm(argv[1], &image, &width, &height);
  size=width*height;


  /* allocate memory for fature maps */
  inten_map = (int *) calloc(size, sizeof(int));
  edge_map = (int *) calloc (size, sizeof(int));
  mag = (int *) calloc (size, sizeof(int));


  /* find edges */
  Edges(width, height, size, edge_map, image, mag);
  

  /* set up intensity map */
  norm_char(inten_map, image, size);


  /* make trees */
  level_size[LEVELS-1]=width;
  for (i=LEVELS-2; i>=0; i--)
    level_size[i]=level_size[i+1]/2;

  for(i=LEVELS-1; i>=0; i--)
  {
    temp=level_size[i]*level_size[i];
    one_tree[i] = (int *) malloc(temp*sizeof(int));
    for(j=0; j<temp; j++)
      one_tree[i][j]=1;
  }

  tree(inten_map, inten_tree, level_size, one_tree);
  tree(mag, mag_tree, level_size, one_tree);
  tree(edge_map, edge_tree, level_size, mag_tree);

for(lev=1; lev<LEVELS; lev++)
{
  el_size=level_size[lev]/11;
  for(orien=0; orien<18; orien+=9)
  {
    sprintf(f_name, "o%d.lev%d.data", orien, lev);
    if (!(out_o=fopen(f_name, "w")))
	printf("Error opening file: %s\n", f_name);
    sprintf(f_name, "m%d.lev%d.data", orien, lev);
    if (!(out_m=fopen(f_name, "w")))
	printf("Error opening file: %s\n", f_name);
    y=el_size+(orien/5)*2*el_size;
    x=el_size+(orien%5)*2*el_size;
    printf("orien:%d x:%d y:%d\n", orien, x, y);
    for(i=y; i<y+el_size; i++)
    {
      for(j=x; j<x+el_size; j++)
      {
        fprintf(out_o, "%d ", edge_tree[lev][i*level_size[lev]+j]);
        fprintf(out_m, "%d ", mag_tree[lev][i*level_size[lev]+j]);
      }
      fprintf(out_o, "\n");
      fprintf(out_m, "\n");
    }  
    fclose(out_o);
    fclose(out_m);
  }
}

  /* find contrast for each level of resolution */
  i=1;
  max_con=0;
  for (c_rad=2; c_rad<=2; c_rad++)
  {
    contrast(inten_tree[i], edge_tree[i], level_size[i],
		&(dbl_map[c_rad-2]), c_rad, &t_max, mag_tree[i]);

    if (t_max>max_con)
    {
      max_con=t_max;
      max_ind=c_rad-2;
      printf("max_ind=%d\n", max_ind);
    }

  }


  /* convert double map to chars for saving */
  c_map = (unsigned char *) malloc(level_size[i]*level_size[i]);
  d_to_c(dbl_map[max_ind], c_map, level_size[i]*level_size[i]);

  /* write results to pgm files */
  strncpy(f_name, argv[1], strlen(argv[1])-3);
  f_name[strlen(argv[1])-3]=0;
  sprintf(f_name, "%sc.pgm", f_name);
  write_pgm(f_name, c_map, level_size[i], level_size[i], max);
 
}

void norm_char(int *to, unsigned char *from, int size)
{
  int i, max=0; 
  double factor;

  for(i=0; i<size; i++)
    if (from[i]>max) max=from[i];

  factor=255.0/max;
  for(i=0; i<size; i++)
    to[i]=from[i]*factor;
}
