#include <stdio.h>
#include <math.h>
#include "letters.h"
#include "pgm.h"
#include "contrast.h"

main()
{
  int max, height, width, size;
  unsigned char *image;
  unsigned char *l_map[3];

  max=read_pgm("bergen1.pgm", &image, &width, &height);
  size=width*height;

  Letters(width, height, size, l_map, image);

  write_pgm("T.pgm", l_map[0], width, height, max);
  write_pgm("L.pgm", l_map[1], width, height, max);
  write_pgm("X.pgm", l_map[2], width, height, max);

}


void Letters (int width, int height, int size, 
		unsigned char **letter_map, unsigned char *image)
{
  unsigned char *T[8], *L[8], *X[2];
  double **temp_map;
  int i, j, radius;

  temp_map = (double **) calloc(3, sizeof(double*));
  for(i=0; i<3; i++)
  {
    letter_map[i] = (unsigned char *) calloc(size, 1);
    temp_map[i] = (double *) calloc(size, sizeof(double));
    for(j=0; j<size; j++)
      temp_map[i][j]=-1;
  }

  printf("read_masks\n");
  radius = read_masks(T, L, X);

  printf("run_convolve T\n");
  run_convolve(image, T, 8, size, temp_map[0], width, radius);
  printf("run_convolve L\n");
  run_convolve(image, L, 8, size, temp_map[1], width, radius);
  printf("run_convolve X\n");
  run_convolve(image, X, 2, size, temp_map[2], width, radius);

  printf("Normalize\n");
  MultipleNormalize(size, letter_map, temp_map, 3);
}

int read_masks(unsigned char **T, unsigned char **L,
		unsigned char **X)
{
  char T_name[30], L_name[30], X_name[30];
  int i, num, w, h;

  strcpy(T_name, "images/letters/T1.pgm");
  strcpy(L_name, "images/letters/L1.pgm");
  strcpy(X_name, "images/letters/X1.pgm");

  num=strlen(X_name)-5;
  printf("num=%d\n", num);

  for(i=0; i<8; i++)
  {
    read_pgm(T_name, &(T[i]), &w, &h);
    read_pgm(L_name, &(L[i]), &w, &h);
    T_name[num]++;
    L_name[num]++;
    if (i<2)
    {
      read_pgm(X_name, &(X[i]), &w, &h);
      X_name[num]++;
    }
  }
  return w;
}

void run_convolve(unsigned char *image, unsigned char **masks, 
		int num, int size, double *map, int width, int radius)
{
  int i, pixel_x, pixel_y;
  double cell, **d_mask;

  d_mask = (double **) calloc(width, sizeof(double *));
  for(i=0; i<width; i++)
    d_mask[i] = (double *) calloc(width, sizeof(double));

  for(i=0; i<num; i++)
  {
    printf("\ti=%d\n", i);
    for(pixel_y=0; pixel_y<width; pixel_y++)
      for(pixel_x=0; pixel_x<width; pixel_x++)
   	d_mask[pixel_y][pixel_x] = masks[i][pixel_y*width+pixel_x];

    for(pixel_y=radius; pixel_y<width-radius; pixel_y++)
      for(pixel_x=radius; pixel_x<width-radius; pixel_x++)
      {
        cell=Convolve(pixel_x, pixel_y, width, radius, d_mask, image);
        if (abs(cell)>map[pixel_y*width+pixel_x])
          map[pixel_y*width+pixel_x] = abs(cell);
      }
  }
}


void MultipleNormalize(int size, unsigned char **e_map, 
			double **t_map, int num_maps)
{
  int i;
  double currMax, maxVal = 0.0;
  
  for(i=0; i< num_maps; i++)
  {
    currMax = Maximum(size, t_map[i]);
    if (currMax > maxVal)
       maxVal=currMax;
  }

  for(i=0; i< num_maps; i++)

    Normalize(i, maxVal, size, e_map, t_map);

  return;
}  



void Normalize (int edge, int maxValue, int size, unsigned 
		char **e_map, double **temp_map)
{
  int i;
  float multFac;

  multFac=255.0/maxValue;

  for(i=0; i<size; i++)
    e_map[edge][i]=(unsigned 
char)(temp_map[edge][i]*multFac);
}



float Maximum(int size, double *temp_map)
{
  int i;
  double max=0;

  for (i=0; i<size; i++)
  {
    if (temp_map[i]>max)
      max=temp_map[i];
  }

  return max;
}




