#include<stdio.h>
#include<stdlib.h>
#include "pgm.h"
#include "edges.h"
#include "contrast.h"
#include "test.h"
#include "globals.h"

main(int argc, char *argv[])
{
  unsigned char *image;	/* for read pgm */
  int width, height, max;
  int imsize;
  int elsize, c_rad;

  unsigned char **edge_map;
  unsigned char *inten_map[2];

  unsigned char ***edge_data;
  unsigned char ***inten_data;

  unsigned char ***edge_tree;
  unsigned char ***inten_tree;

  double ***orien_con_map;
  double ***inten_con_map;

  double **c_map_dbl;
  unsigned char *c_map_char;

  double **gaus_c;
  double **gaus_s;

  int i, j, level;
  int level_size[Levels];
  double max_con, *temp_max, feature_max;
  int max_ind;
  int index;
  char f_name[20];

  read_data_file(argv[1]);
 
  printf("\n%s\n", Image_Name);
  
  /* read pgm file */
  max = read_pgm(Image_Name, &image, &width, &height);
  imsize=width*height;

  /* set array of size of each hierarchy level */
  level_size[Levels-1]=width;
  for (i=Levels-2; i>=0; i--)
    level_size[i]=level_size[i+1]/2;

  /* allocate storage for double version of contrast map */
  c_map_dbl = (double **) calloc(Levels, sizeof(double *));
  for(i=0; i<Levels; i++)
    c_map_dbl[i] = (double *) calloc(imsize, sizeof(double));

  /* Allocate storage for maximum values at each radius value */
  temp_max = (double *) calloc(Max_Radius-Min_Radius+1, sizeof(double));

  /* Allocate storage for Gaussian masks */
  gaus_c=(double **) calloc(2*Max_Radius+1, sizeof(double*));
  for (i=0; i<2*Max_Radius+1; i++)
    gaus_c[i]=(double *) calloc(2*Max_Radius+1, sizeof(double));

  gaus_s=(double **) calloc(2*3*Max_Radius+1, sizeof(double*));
  for (i=0; i<2*3*Max_Radius+1; i++)
    gaus_s[i]=(double *) calloc(2*3*Max_Radius+1, sizeof(double));

  level=1;


  if (I_Contrast)
  {
    inten_data = (unsigned char ***) calloc(Levels, sizeof(unsigned char **));
    for (i=0; i<Levels; i++)
      inten_data[i] = (unsigned char **) calloc(2, sizeof(unsigned char *));

    inten_tree = (unsigned char ***) calloc(2, sizeof(unsigned char **));
    inten_tree[0] = (unsigned char **) calloc(Levels, sizeof(unsigned char *));
    inten_tree[1] = (unsigned char **) calloc(Levels, sizeof(unsigned char *));
   
    inten_map[0] = (unsigned char *) malloc(imsize);
    inten_map[1] = (unsigned char *) malloc(imsize);

    inten_con_map = (double ***) calloc(Levels, sizeof(double **));

    /* set up intensity maps */
    norm_char(inten_map[0], image, imsize);
    negate(inten_map[1], inten_map[0], imsize);

    /* make intensity trees */
    tree(inten_map[0], inten_tree[0], level_size);
    tree(inten_map[1], inten_tree[1], level_size);

    /* find contrast for each level of resolution */
    for (i=0; i<Levels; i++)
    {
      inten_data[i][0]=inten_tree[0][i];
      inten_data[i][1]=inten_tree[1][i];
    }

    /* calculate intensity contrast */
    for (c_rad=Min_Radius; c_rad<=Max_Radius; c_rad++)
    {
      index=c_rad-Min_Radius;
      feature_max=contrast(inten_data[level], 
		&(inten_con_map[index]), 
		gaus_c, gaus_s,
		level_size[level], c_rad, 2);

      if (feature_max>temp_max[index]) temp_max[index]=feature_max;
    }
  }


  if (O_Contrast)
  {
    edge_map = (unsigned char **) 
			calloc(Num_Orientations, sizeof(unsigned char *));

    edge_data = (unsigned char ***) calloc(Levels, sizeof(unsigned char **));
    for (i=0; i<Levels; i++)
      edge_data[i] = (unsigned char **)
			calloc(Num_Orientations, sizeof(unsigned char *));

    edge_tree = (unsigned char ***)
			calloc(Num_Orientations, sizeof(unsigned char **));
    for(i=0; i<Num_Orientations; i++)
      edge_tree[i] = (unsigned char **) calloc(Levels, sizeof(unsigned char *));

    orien_con_map = (double ***) malloc(Levels*sizeof(double **));

    /* find edges (edge_maps) */
    Edges(width, height, imsize, edge_map, image);

    /* make orientation trees */
    for (i=0; i<Num_Orientations; i++)
      tree(edge_map[i], edge_tree[i], level_size);

    /* find contrast for each level of resolution */
    for (i=0; i<Levels; i++)
    {
      for (j=0; j<Num_Orientations; j++)
        edge_data[i][j]=edge_tree[j][i];
    } 

    /* calculate orientation contrast */
    for (c_rad=Min_Radius; c_rad<=Max_Radius; c_rad++)
    {
      index=c_rad-Min_Radius;
      feature_max=contrast(edge_data[level], 
		&(orien_con_map[index]),
		gaus_c, gaus_s,
		level_size[level], c_rad, Num_Orientations);

      if (feature_max>temp_max[index]) temp_max[index]=feature_max;
    }
  }


  max_con=0;

  for (c_rad=Min_Radius; c_rad<=Max_Radius; c_rad++)
  {
    index=c_rad-Min_Radius;     
    printf("iter: %d\n", c_rad);
    combine_maps(temp_max[index], c_map_dbl[index], orien_con_map, 
		inten_con_map, index, level_size[level]*level_size[level]);

    if (temp_max[index]>max_con)
    {
      max_con=temp_max[index];
      max_ind=c_rad-Min_Radius;
      printf("max_ind=%d\n", max_ind);
    }
  }


  /* convert double map to chars for saving */
  c_map_char = (unsigned char *) malloc(level_size[level]*level_size[level]);
  d_to_c(c_map_dbl[max_ind], c_map_char, level_size[level]*level_size[level]);

  /* write results to pgm files */
  strncpy(f_name, Image_Name, strlen(Image_Name)-3);
  f_name[strlen(Image_Name)-3]=0;
  sprintf(f_name, "%sc.pgm", f_name);
  write_pgm(f_name, c_map_char, level_size[level], level_size[level], max);
 
}

void norm_char(unsigned char *to, unsigned char *from, int size)
{
  int i, max=0; 
  double factor;

  for(i=0; i<size; i++)
    if (from[i]>max) max=from[i];

  factor=255.0/max;
  for(i=0; i<size; i++)
    to[i]=from[i]*factor;
}

void negate(unsigned char *to, unsigned char *from, int size)
{
  int j;

  for(j=0; j<size; j++)
    to[j]=255-from[j];
}
